/*
 * Copyright (c) 1995, the EUROPAGATE consortium (see below).
 *
 * The EUROPAGATE consortium members are:
 *
 *    University College Dublin
 *    Danmarks Teknologiske Videnscenter
 *    An Chomhairle Leabharlanna
 *    Consejo Superior de Investigaciones Cientificas
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation, in whole or in part, for any purpose, is hereby granted,
 * provided that:
 *
 * 1. This copyright and permission notice appear in all copies of the
 * software and its documentation. Notices of copyright or attribution
 * which appear at the beginning of any file must remain unchanged.
 *
 * 2. The names of EUROPAGATE or the project partners may not be used to
 * endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 3. Users of this software (implementors and gateway operators) agree to
 * inform the EUROPAGATE consortium of their use of the software. This
 * information will be used to evaluate the EUROPAGATE project and the
 * software, and to plan further developments. The consortium may use
 * the information in later publications.
 * 
 * 4. Users of this software agree to make their best efforts, when
 * documenting their use of the software, to acknowledge the EUROPAGATE
 * consortium, and the role played by the software in their work.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED, OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 * IN NO EVENT SHALL THE EUROPAGATE CONSORTIUM OR ITS MEMBERS BE LIABLE
 * FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF
 * ANY KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA
 * OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND
 * ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
/* Gateway utility
 * Europagate, 1995
 *
 * $Log: strqueue.c,v $
 * Revision 1.3  1995/05/18 12:03:41  adam
 * Bug fix in gipc_open: didn't catch EINTR.
 * Memory leak fix in strqueue.c.
 *
 * Revision 1.2  1995/05/16  09:40:55  adam
 * LICENSE.
 *
 * Revision 1.1  1995/03/28  11:42:41  adam
 * First version of string-queue utility.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>

#include <strqueue.h>

struct str_entry {
    char *buf;
    struct str_entry *prev;
};

struct str_queue *str_queue_mk (void)
{
    struct str_queue *sq;

    sq = malloc (sizeof(*sq));
    if (!sq)
        return NULL;
    sq->first = NULL;
    sq->last = NULL;
    return sq;
}

void str_queue_rm (struct str_queue **sq)
{
    while (str_queue_deq (*sq, NULL, 0))
        ;
    free (*sq);
    *sq = NULL;
}

char *str_queue_enq (struct str_queue *sq, const char *msg)
{
    struct str_entry *n;

    assert (sq);
    assert (msg);
    n = malloc (sizeof(*n));
    if (!n)
        return NULL;
    n->buf = malloc (strlen(msg)+1);
    if (!n->buf)
    {
        free (n);
        return NULL;
    }
    strcpy (n->buf, msg);
    n->prev = NULL;
    if (!sq->first)
        sq->last = n;
    else
        sq->first->prev = n;
    sq->first = n;
    return n->buf;
}

char *str_queue_get (struct str_queue *sq, int index)
{
    struct str_entry *n;

    assert (sq);
    for (n = sq->last; n && index > 0; --index)
        n = n->prev;
    if (!n)
        return NULL;
    return n->buf;
}

int str_queue_deq (struct str_queue *sq, char *dst, int max)
{
    struct str_entry *n;

    assert (sq);
    if (!(n = sq->last))
        return 0;
    sq->last = n->prev;
    if (!sq->last)
        sq->first = NULL;
    if (dst && max > 1)
    {
        strncpy (dst, n->buf, max-1);
        dst[max-1] = '\0';
    }
    free (n->buf);
    free (n);
    return 1;
}
